/**
 * ===============================================================
 * A tiny MML parser (Version 0.5.0)
 * ===============================================================
 * Copyright (c) 2014-2015 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <mml.h>
#include "note.h"
#include "song.h"

#define SW_START 2
#define SW_TOUCH 3
#define SW_GOAL 4
#define LED_OUT 7

NOTE note;
MML mml;
MML_OPTION mml_opt;

float START_TIME;
float GOAL_TIME;
boolean TOUCH_STATE = true;

/* MML perser setup ST */
static void mml_callback(MML_INFO *p, void *extobj)
{
  switch (p->type) {
    case MML_TYPE_TEMPO:
      {
        MML_ARGS_TEMPO *args = &(p->args.tempo);
        note_init(&note, args->value, mml_opt.bticks);
      }
      break;
    case MML_TYPE_NOTE:
      {
        MML_ARGS_NOTE *args = &(p->args.note);
        note_tone(&note, args->number, args->ticks);
      }
      break;
    case MML_TYPE_REST:
      {
        MML_ARGS_REST *args = &(p->args.rest);
        note_rest(&note, args->ticks);
      }
      break;
  }
}

void wave_start()
{
    mml_setup(&mml, &mml_opt, (char *)wave_start_text);
    while (mml_fetch(&mml) == MML_RESULT_OK) {  } 
}
void wave_touch()
{
    mml_setup(&mml, &mml_opt, (char *)wave_touch_text);
    while (mml_fetch(&mml) == MML_RESULT_OK) {  } 
}
void wave_goal()
{
    mml_setup(&mml, &mml_opt, (char *)wave_goal_text);
    while (mml_fetch(&mml) == MML_RESULT_OK) {  } 
}
/* MML perser setup ED */


/* Start,Touch,Goal ST */
void START()
{
    if(TOUCH_STATE == true)
    {
        TOUCH_STATE = false;
        START_TIME = millis()*1.000;
        digitalWrite(LED_OUT, LOW);
        Serial.println("* * * * S T A R T !! (-v-) * * * * *");
        Serial.println("    Good luck ... ");
        wave_start();
        delay(500);
        digitalWrite(SW_START,HIGH);
    }
}

void TOUCH()
{
    if(TOUCH_STATE == false)
    {
        TOUCH_STATE = true;
        digitalWrite(LED_OUT, HIGH);
        GOAL_TIME = float (millis() - START_TIME)/1000.000;
        Serial.println("x x x x C R A S H E D !! (T_T) x x x")  ;
        Serial.print("    Total time : ");
        Serial.println(GOAL_TIME);
        wave_touch();
        digitalWrite(SW_TOUCH,HIGH);
    }
}

void GOAL()
{
    TOUCH_STATE = true;
    GOAL_TIME =float (millis()- START_TIME)/1000.000;
    Serial.println("o o o o G O A L !! (^v^) o o o o o o");
    Serial.print("    Total time : ");
    Serial.println(GOAL_TIME);
    wave_goal();
    delay(500);
    digitalWrite(LED_OUT, HIGH);
    digitalWrite(SW_GOAL,HIGH);
}
/* Start,Touch,Goal ED */


/* Arduino UNO setup ST */
void setup()
{
    Serial.begin(9600);
    pinMode(SW_START,INPUT_PULLUP);
    pinMode(SW_TOUCH,INPUT_PULLUP);
    pinMode(SW_GOAL,INPUT_PULLUP);
    pinMode(LED_OUT,OUTPUT);
    digitalWrite(LED_OUT,HIGH);
    TOUCH_STATE = true;
    /* Initialize the MML and note  module.  */
    mml_init(&mml, mml_callback, 0);
    MML_OPTION_INITIALIZER_DEFAULT(&mml_opt);
    int tempo_default = 160;
    note_init(&note, tempo_default, mml_opt.bticks);
}
/* Arduino UNO setup ED */
/* Loop function ST */
void loop() 
{
    if(digitalRead(SW_TOUCH) == LOW && TOUCH_STATE == false)
    {
        TOUCH();
    }
    if(digitalRead(SW_START) == LOW && TOUCH_STATE == true)
    {
        START();
    }
    if(digitalRead(SW_GOAL) == LOW  && TOUCH_STATE == false)
    {
        GOAL();
    }
    else
    {
        delay(50);
    }
}
/* Loop function ED */

